// +-------------------------------- DISCLAIMER ---------------------------------+
// |                                                                             |
// | This application program is provided to you free of charge as an example.   |
// | Despite the considerable efforts of Euresys personnel to create a usable    |
// | example, you should not assume that this program is error-free or suitable  |
// | for any purpose whatsoever.                                                 |
// |                                                                             |
// | EURESYS does not give any representation, warranty or undertaking that this |
// | program is free of any defect or error or suitable for any purpose. EURESYS |
// | shall not be liable, in contract, in torts or otherwise, for any damages,   |
// | loss, costs, expenses or other claims for compensation, including those     |
// | asserted by third parties, arising out of or in connection with the use of  |
// | this program.                                                               |
// |                                                                             |
// +-----------------------------------------------------------------------------+
//
// ImageProcessing.cpp: implementation of the ImageProcessing class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "FuseInspection.h"
#include "ImageProcessing.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ImageProcessing::ImageProcessing()
{

}

ImageProcessing::~ImageProcessing()
{

}

 void ImageProcessing::Initialize()
{
	try
	{
		/// Load the EasyGauge model file 
		_worldShape.Load("..\\Resources\\Model\\Fuse.GGE", true);

		/// Load the EasyFind model file 
		_finder.Load("..\\Resources\\Model\\Fuse.FND");
	}
	catch(EException exc)
	{
		::MessageBox(NULL, exc.What().c_str(), "Error", MB_OK | MB_ICONERROR);
	}
}

void ImageProcessing::Inspect(LPCTSTR filename)
{
	// Reset the flags
	_bBlown= false;
	_bFound= false;

	try
	{
		// Load the image for inspection
		_srcImg.Load(filename);

		// If the EasyFind model file loading failed, exit
		if(!_finder.GetLearningDone())
			return;

		// Find the Fuse
		_foundPattern = _finder.Find(&_srcImg);

		// If no instance has been found, exit
		if (_foundPattern.size() == 0)
		    return;

		// Retrieve the found pattern center coordinates (in Sensor space)
		EPoint sensorFrameCenter(_foundPattern[0].GetCenter().GetX(), _foundPattern[0].GetCenter().GetY());

		// As the field of view is calibrated, it is necessary to convert the
		// instance coordinates from the Sensor space (pixels) to the World space (mm)
		EPoint worldFrameCenter;
		worldFrameCenter = _worldShape.SensorToWorld(sensorFrameCenter);

		// Move the FrameShape to the relevant location
		EFrameShape* frameShape;
		frameShape = (EFrameShape*)_worldShape.GetShapeNamed("EFrameShape1");
		frameShape->SetCenter(worldFrameCenter);
		frameShape->SetAngle((-1.0f) * _foundPattern[0].GetAngle());

		// Perform the measurement
		_worldShape.Process(&_srcImg, true);

		// Retrieve the number of valid points
		ELineGauge* line;
		line = (ELineGauge*)_worldShape.GetShapeNamed("ELineGauge1");

		int numSamples = line->GetNumSamples();
		int numValidSamples = line->GetNumValidSamples();

		// If less than 75% of the sample points are valid, then the fuse is blown
		if ((numSamples - numValidSamples) > (numSamples / 4) + 1)
		{
		    // Update the flag
		    _bBlown = true;
		}

		// Update the flag
		_bFound= true;
	}
	catch(EException exc)
	{
		::MessageBox(NULL, exc.What().c_str(), "Error", MB_OK | MB_ICONERROR);
	}
}

void ImageProcessing::Draw(HDC hdc)
{

	try
	{
		/// Exit if no image has been loaded
		if (_srcImg.IsVoid())
			return;

		/// Draw the source image
		_srcImg.Draw(hdc);

		// If no fuse was found, exit
		if(!_bFound)
			return;

		CString strLine;
		ELineGauge* line;

		// Draw the gauges measuring the fuse leads
		for (int i = 3; i < 11; i++)
		{
		    strLine.Format("ELineGauge%d", i);

			line = (ELineGauge*)_worldShape.GetShapeNamed((std::string)strLine);
		    line->Draw(hdc, EDrawingMode_Nominal);
		    line->Draw(hdc, EDrawingMode_Actual);
		}

		// Draw green lines if the fuse is not blown
		if (_bBlown == false)
		{
		    line = (ELineGauge*)_worldShape.GetShapeNamed("ELineGauge1");
		    line->Draw(hdc, EDrawingMode_Actual);

		    line = (ELineGauge*)_worldShape.GetShapeNamed("ELineGauge2");
		    line->Draw(hdc, EDrawingMode_Actual);
		}
		// Draw red lines if the fuse is blown
		else if (_bBlown == true)
		{
			// Specify the color for drawing
		    ERGBColor redColor(255, 0, 0);
			
		    line = (ELineGauge*)_worldShape.GetShapeNamed("ELineGauge1");
		    line->Draw(hdc, redColor, EDrawingMode_Actual);
			
		    line = (ELineGauge*)_worldShape.GetShapeNamed("ELineGauge2");
		    line->Draw(hdc, redColor, EDrawingMode_Actual);
		}
 	}
	catch(EException exc)
	{
		::MessageBox(NULL, exc.What().c_str(), "Error", MB_OK | MB_ICONERROR);
	}

}
