using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Windows.Forms;

// Open eVision Namespace
using Euresys.Open_eVision_1_2;


namespace FuseInspection
{
 
    class ImageProcessing
    {
        // Source image        
        private EImageBW8 _srcImg = new EImageBW8();

        // Geometrical pattern matching tool
        private EPatternFinder _finder = new EPatternFinder();
        private EFoundPattern[] _found = null;

        // EWorldShape which holds the measurement tools
        private EWorldShape _worldShape = new EWorldShape();

        // Flags
        private bool _bBlown;   // true if the fuse is blown
        private bool _bFound;   // true if a fuse has been found


        /// <summary>
        /// Constructor
        /// </summary>
        public ImageProcessing()
        {

        }
        
        /// <summary>
        /// Destructor
        /// </summary>
        ~ImageProcessing()
        {
        }

        /// <summary>
        /// Initialization
        /// </summary>
        public void Initialize()
        {
            try
            {
                // Load the EasyFind model file 
                _finder.Load(@"..\..\..\Resources\Model\Fuse.FND", true);

                // Load the EasyGauge model file 
                _worldShape.Load(@"..\..\..\Resources\Model\Fuse.GGE", true);

            }
            catch (Euresys.Open_eVision_1_2.EException exc)
            {
                MessageBox.Show(exc.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Inspection.
        /// </summary>
        public void Inspect(string filename)
        {
            // Reset the flags
            _bBlown = false;
            _bFound = false;

            try
            {
                // Load the image for inspection
                _srcImg.Load(filename);

                // If the EasyFind model file loading failed, exit
                if (!_finder.LearningDone)
                    return;
                
                // Find the Fuse
                _found = _finder.Find(_srcImg);

                // If no instance has been found, exit
                if (_found.Length == 0)
                    return;

                // Retrieve the found pattern center coordinates (in Sensor space)
                EPoint sensorFrameCenter = new EPoint(_found[0].Center.X, _found[0].Center.Y);

                // As the field of view is calibrated, it is necessary to convert the
                // instance coordinates from the Sensor space (pixels) to the World space (mm)
                EPoint worldFrameCenter = new EPoint();
                worldFrameCenter = _worldShape.SensorToWorld(sensorFrameCenter);

                // Move the FrameShape to the relevant location
                EFrameShape frameShape = null;
                frameShape = (EFrameShape)_worldShape.GetShapeNamed("EFrameShape1");
                frameShape.SetCenterXY(worldFrameCenter.X, worldFrameCenter.Y);
                frameShape.Angle = (-1.0f) * _found[0].Angle;

                // Perform the measurement
                _worldShape.Process(_srcImg, true);

                // Retrieve the number of valid points
                ELineGauge line = null;
                line = (ELineGauge)_worldShape.GetShapeNamed("ELineGauge1");

                int numSamples = line.NumSamples;
                int numValidSamples = line.NumValidSamples;

                // If less than 75% of the sample points are valid, then the fuse is blown
                if ((numSamples - numValidSamples) > (numSamples / 4) + 1)
                {
                    // Update the flag
                    _bBlown = true;
                }

                // Update the flag
                _bFound = true;
            }
            catch (Euresys.Open_eVision_1_2.EException exc)
            {
                MessageBox.Show(exc.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Drawing.
        /// </summary>
        public void Draw(Graphics g)
        {
            try
            {
                // Exit if no image has been loaded
                if (_srcImg.IsVoid)
                    return;

                // Draw the inspected image
                _srcImg.Draw(g);

                // If no fuse was found, exit
                if (_bFound == false)
                    return;

                string strLine;
                string strTmp;
                ELineGauge line = null;

                // Draw the gauges measuring the fuse leads
                for (int i = 3; i < 11; i++)
                {
                    strLine = "ELineGauge";
                    strTmp = strLine + i;

                    line = (ELineGauge)_worldShape.GetShapeNamed(strTmp);
                    line.Draw(g, EDrawingMode.Nominal);
                    line.Draw(g, EDrawingMode.Actual);
                }

                // Draw green lines if the fuse is not blown
                if (_bBlown == false)
                {
                    line = (ELineGauge)_worldShape.GetShapeNamed("ELineGauge1");
                    line.Draw(g, EDrawingMode.Actual);

                    line = (ELineGauge)_worldShape.GetShapeNamed("ELineGauge2");
                    line.Draw(g, EDrawingMode.Actual);
                }
                // Draw red lines if the fuse is blown
                else if (_bBlown == true)
                {
                    // Specify the color for drawing
			        ERGBColor redColor= new ERGBColor(255, 0, 0); 

                    line = (ELineGauge)_worldShape.GetShapeNamed("ELineGauge1");
                    line.Draw (g, redColor, EDrawingMode.Actual);

                    line = (ELineGauge)_worldShape.GetShapeNamed("ELineGauge2");
                    line.Draw(g, redColor, EDrawingMode.Actual);
                }
            }
            catch (Euresys.Open_eVision_1_2.EException exc)
            {
                MessageBox.Show(exc.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }

}
